@@ -1,35 +1,23 @@
+// SPDX-License-Identifier: GPL-2.0
+
 /*
- * Copyright (C) 2013 Mauro Carvalho Chehab <mchehab+redhat@kernel.org>
+ * Copyright (C) 2013 Mauro Carvalho Chehab <mchehab+huawei@kernel.org>
  *
- * The code below were adapted from Andi Kleen/Intel/SuSe mcelog code,
+ * The code below were adapted from Andi Kleen/Intel/SUSE mcelog code,
  * released under GNU Public General License, v.2
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
-*/
+ */
 
 #include <errno.h>
 #include <fcntl.h>
-#include <string.h>
 #include <stdio.h>
-#include <unistd.h>
-#include <sys/types.h>
+#include <string.h>
 #include <sys/stat.h>
+#include <sys/types.h>
+#include <unistd.h>
 
+#include "bitfield.h"
 #include "ras-logger.h"
 #include "ras-mce-handler.h"
-#include "bitfield.h"
 
 #define MCE_THERMAL_BANK	(MCE_EXTENDED_BANK + 0)
 #define MCE_TIMEOUT_BANK        (MCE_EXTENDED_BANK + 90)
@@ -57,8 +45,7 @@
 #define BUS_PP_MASK      0x600 /*bit 9, bit 10*/
 #define BUS_PP_SHIFT     0x9
 
-#define MCG_TES_P       (1ULL<<11)   /* Yellow bit cache threshold supported */
-
+#define MCG_TES_P       BIT_ULL(11)   /* Yellow bit cache threshold supported */
 
 static char *TT[] = {
 	"Instruction",
@@ -76,8 +63,8 @@ static char *LL[] = {
 
 static struct {
 	uint8_t value;
-	char* str;
-} RRRR [] = {
+	char *str;
+} RRRR[] = {
 	{0, "Generic"},
 	{1, "Read"},
 	{2, "Write" },
@@ -121,7 +108,7 @@ static char *mca_msg[] = {
 static char *tracking_msg[] = {
 	[1] = "green",
 	[2] = "yellow",
-	[3] ="res3"
+	[3] = "res3"
 };
 
 static const char *arstate[4] = {
@@ -152,23 +139,28 @@ static void decode_memory_controller(struct mce_event *e, uint32_t status)
 	char channel[30];
 
 	if ((status & 0xf) == 0xf)
-		sprintf(channel, "unspecified");
+		snprintf(channel, sizeof(channel), "unspecified");
 	else
-		sprintf(channel, "%u", status & 0xf);
+		snprintf(channel, sizeof(channel), "%u", status & 0xf);
 
 	mce_snprintf(e->error_msg, "MEMORY CONTROLLER %s_CHANNEL%s_ERR",
-		    mmm_mnemonic[(status >> 4) & 7], channel);
+		     mmm_mnemonic[(status >> 4) & 7], channel);
 	mce_snprintf(e->error_msg, "Transaction: %s",
-		    mmm_desc[(status >> 4) & 7]);
+		     mmm_desc[(status >> 4) & 7]);
 }
 
 static void decode_termal_bank(struct mce_event *e)
 {
 	if (e->status & 1) {
-		mce_snprintf(e->mcgstatus_msg, "Processor %d heated above trip temperature. Throttling enabled.", e->cpu);
-		mce_snprintf(e->user_action, "Please check your system cooling. Performance will be impacted");
+		mce_snprintf(e->mcgstatus_msg,
+			     "Processor %d heated above trip temperature. Throttling enabled.",
+			     e->cpu);
+		mce_snprintf(e->user_action,
+			     "Please check your system cooling. Performance will be impacted");
 	} else {
-		mce_snprintf(e->error_msg, "Processor %d below trip temperature. Throttling disabled", e->cpu);
+		mce_snprintf(e->error_msg,
+			     "Processor %d below trip temperature. Throttling disabled",
+			     e->cpu);
 	}
 }
 
@@ -191,14 +183,14 @@ static void decode_mcg(struct mce_event *e)
 
 static void bank_name(struct mce_event *e)
 {
-	char *buf = e->bank_name;
-
 	switch (e->bank) {
 	case MCE_THERMAL_BANK:
-		strcpy(buf, "THERMAL EVENT");
+		strscpy(e->bank_name, "THERMAL EVENT", sizeof(e->bank_name));
 		break;
 	case MCE_TIMEOUT_BANK:
-		strcpy(buf, "Timeout waiting for exception on other CPUs");
+		strscpy(e->bank_name,
+			"Timeout waiting for exception on other CPUs",
+			sizeof(e->bank_name));
 		break;
 	default:
 		break;
@@ -207,12 +199,11 @@ static void bank_name(struct mce_event *e)
 
 static char *get_RRRR_str(uint8_t rrrr)
 {
-	unsigned i;
+	unsigned int i;
 
 	for (i = 0; i < ARRAY_SIZE(RRRR); i++) {
-		if (RRRR[i].value == rrrr) {
+		if (RRRR[i].value == rrrr)
 			return RRRR[i].str;
-		}
 	}
 
 	return "UNKNOWN";
@@ -220,7 +211,7 @@ static char *get_RRRR_str(uint8_t rrrr)
 
 #define decode_attr(arr, val) ({				\
 	char *__str;						\
-	if ((unsigned)(val) >= ARRAY_SIZE(arr))			\
+	if ((unsigned int)(val) >= ARRAY_SIZE(arr))			\
 		__str = "UNKNOWN";				\
 	else							\
 		__str = (arr)[val];				\
@@ -248,22 +239,17 @@ static void decode_mca(struct mce_event *e, uint64_t track, int *ismemerr)
 			     decode_attr(LL, mca & 3));
 	} else if (test_prefix(4, mca)) {
 		mce_snprintf(e->mcastatus_msg, "%s TLB %s Error",
-				decode_attr(TT, (mca & TLB_TT_MASK) >> TLB_TT_SHIFT),
-				decode_attr(LL, (mca & TLB_LL_MASK) >> TLB_LL_SHIFT));
+			     decode_attr(TT, (mca & TLB_TT_MASK) >> TLB_TT_SHIFT),
+			     decode_attr(LL, (mca & TLB_LL_MASK) >> TLB_LL_SHIFT));
 	} else if (test_prefix(8, mca)) {
-		unsigned typenum = (mca & CACHE_TT_MASK) >> CACHE_TT_SHIFT;
-		unsigned levelnum = (mca & CACHE_LL_MASK) >> CACHE_LL_SHIFT;
+		unsigned int typenum = (mca & CACHE_TT_MASK) >> CACHE_TT_SHIFT;
+		unsigned int levelnum = (mca & CACHE_LL_MASK) >> CACHE_LL_SHIFT;
 		char *type = decode_attr(TT, typenum);
 		char *level = decode_attr(LL, levelnum);
+
 		mce_snprintf(e->mcastatus_msg,
 			     "%s CACHE %s %s Error", type, level,
-			     get_RRRR_str((mca & CACHE_RRRR_MASK) >>
-					      CACHE_RRRR_SHIFT));
-#if 0
-		/* FIXME: We shouldn't mix parsing with actions */
-		if (track == 2)
-			run_yellow_trigger(e->cpu, typenum, levelnum, type, level, e->socket);
-#endif
+			     get_RRRR_str((mca & CACHE_RRRR_MASK) >> CACHE_RRRR_SHIFT));
 	} else if (test_prefix(10, mca)) {
 		if (mca == 0x400)
 			mce_snprintf(e->mcastatus_msg,
@@ -282,8 +268,9 @@ static void decode_mca(struct mce_event *e, uint64_t track, int *ismemerr)
 	} else if (test_prefix(7, mca)) {
 		decode_memory_controller(e, mca);
 		*ismemerr = 1;
-	} else
+	} else {
 		mce_snprintf(e->mcastatus_msg, "Unknown Error %x", mca);
+	}
 }
 
 static void decode_tracking(struct mce_event *e, uint64_t track)
@@ -313,15 +300,13 @@ static void decode_mci(struct mce_event *e, int *ismemerr)
 	else
 		mce_snprintf(e->mcistatus_msg, "Corrected_error");
 
-
 	if (e->status & MCI_STATUS_EN)
 		mce_snprintf(e->mcistatus_msg, "Error_enabled");
 
-
 	if (e->status & MCI_STATUS_PCC)
 		mce_snprintf(e->mcistatus_msg, "Processor_context_corrupt");
 
-	if (e->status & (MCI_STATUS_S|MCI_STATUS_AR))
+	if (e->status & (MCI_STATUS_S | MCI_STATUS_AR))
 		mce_snprintf(e->mcistatus_msg, "%s",
 			     arstate[(e->status >> 55) & 3]);
 
@@ -350,14 +335,14 @@ int parse_intel_event(struct ras_events *ras, struct mce_event *e)
 
 	/* Check if the error is at the memory controller */
 	if (((e->status & 0xffff) >> 7) == 1) {
-		unsigned corr_err_cnt;
+		unsigned int corr_err_cnt;
 
 		corr_err_cnt = EXTRACT(e->status, 38, 52);
 		mce_snprintf(e->mc_location, "n_errors=%d", corr_err_cnt);
 	}
 
 	if (test_prefix(11, (e->status & 0xffffL))) {
-		switch(mce->cputype) {
+		switch (mce->cputype) {
 		case CPU_P6OLD:
 			p6old_decode_model(e);
 			break;
@@ -375,7 +360,7 @@ int parse_intel_event(struct ras_events *ras, struct mce_event *e)
 			break;
 		}
 	}
-	switch(mce->cputype) {
+	switch (mce->cputype) {
 	case CPU_NEHALEM:
 		nehalem_decode_model(e);
 		break;
@@ -415,6 +400,7 @@ int parse_intel_event(struct ras_events *ras, struct mce_event *e)
 	case CPU_ICELAKE_DE:
 	case CPU_TREMONT_D:
 	case CPU_SAPPHIRERAPIDS:
+	case CPU_EMERALDRAPIDS:
 		i10nm_decode_model(mce->cputype, ras, e);
 	default:
 		break;
@@ -432,7 +418,7 @@ static int domsr(int cpu, int msr, int bit)
 	unsigned long long data;
 	int fd;
 
-	sprintf(fpath, "/dev/cpu/%d/msr", cpu);
+	snprintf(fpath, sizeof(fpath), "/dev/cpu/%d/msr", cpu);
 	fd = open(fpath, O_RDWR);
 	if (fd == -1) {
 		switch (errno) {
@@ -446,18 +432,18 @@ static int domsr(int cpu, int msr, int bit)
 			return -EINVAL;
 		}
 	}
-	if (pread(fd, &data, sizeof data, msr) != sizeof data) {
+	if (pread(fd, &data, sizeof(data), msr) != sizeof(data)) {
 		log(ALL, LOG_ERR,
 		    "Cannot read MSR_ERROR_CONTROL from %s\n", fpath);
 		return -EINVAL;
 	}
 	data |= bit;
-	if (pwrite(fd, &data, sizeof data, msr) != sizeof data) {
+	if (pwrite(fd, &data, sizeof(data), msr) != sizeof(data)) {
 		log(ALL, LOG_ERR,
 		    "Cannot write MSR_ERROR_CONTROL to %s\n", fpath);
 		return -EINVAL;
 	}
-	if (pread(fd, &data, sizeof data, msr) != sizeof data) {
+	if (pread(fd, &data, sizeof(data), msr) != sizeof(data)) {
 		log(ALL, LOG_ERR,
 		    "Cannot re-read MSR_ERROR_CONTROL from %s\n", fpath);
 		return -EINVAL;
@@ -471,7 +457,7 @@ static int domsr(int cpu, int msr, int bit)
 	return 0;
 }
 
-int set_intel_imc_log(enum cputype cputype, unsigned ncpus)
+int set_intel_imc_log(enum cputype cputype, unsigned int ncpus)
 {
 	int cpu, msr, bit, rc;
 
