@@ -1,49 +1,41 @@
+// SPDX-License-Identifier: GPL-2.0-or-later
+
 /*
- * Copyright (C) 2013 Mauro Carvalho Chehab <mchehab+redhat@kernel.org>
- *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
- * (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
-*/
+ * Copyright (C) 2013 Mauro Carvalho Chehab <mchehab+huawei@kernel.org>
+ */
+
 #include <dirent.h>
 #include <errno.h>
 #include <fcntl.h>
 #include <limits.h>
+#include <signal.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <unistd.h>
+#include <sys/poll.h>
+#include <sys/signalfd.h>
 #include <sys/stat.h>
 #include <sys/types.h>
-#include <sys/poll.h>
-#include <signal.h>
-#include <sys/signalfd.h>
-#include <linux/version.h>
-#include <traceevent/kbuffer.h>
 #include <traceevent/event-parse.h>
-#include "ras-mc-handler.h"
+#include <traceevent/kbuffer.h>
+#include <unistd.h>
+
 #include "ras-aer-handler.h"
-#include "ras-non-standard-handler.h"
 #include "ras-arm-handler.h"
-#include "ras-mce-handler.h"
-#include "ras-extlog-handler.h"
+#include "ras-cpu-isolation.h"
+#include "ras-cxl-handler.h"
 #include "ras-devlink-handler.h"
 #include "ras-diskerror-handler.h"
-#include "ras-memory-failure-handler.h"
-#include "ras-record.h"
+#include "ras-events.h"
+#include "ras-extlog-handler.h"
 #include "ras-logger.h"
+#include "ras-mce-handler.h"
+#include "ras-mc-handler.h"
+#include "ras-memory-failure-handler.h"
+#include "ras-non-standard-handler.h"
 #include "ras-page-isolation.h"
-#include "ras-cpu-isolation.h"
+#include "ras-record.h"
+#include "trigger.h"
 
 /*
  * Polling time, if read() doesn't block. Currently, trace_pipe_raw never
@@ -59,16 +51,25 @@
 	#define ENDIAN KBUFFER_ENDIAN_BIG
 #endif
 
+char *choices_disable;
+
+static const struct event_trigger event_triggers[] = {
+	{ "mc_event", &mc_event_trigger_setup },
+#ifdef HAVE_MEMORY_FAILURE
+	{ "memory_failure_event", &mem_fail_event_trigger_setup },
+#endif
+};
+
 static int get_debugfs_dir(char *tracing_dir, size_t len)
 {
 	FILE *fp;
 	char line[MAX_PATH + 1 + 256];
 	char *p, *type, *dir;
 
-	fp = fopen("/proc/mounts","r");
+	fp = fopen("/proc/mounts", "r");
 	if (!fp) {
 		log(ALL, LOG_INFO, "Can't open /proc/mounts");
-		return errno;
+		return -errno;
 	}
 
 	do {
@@ -89,26 +90,60 @@ static int get_debugfs_dir(char *tracing_dir, size_t len)
 
 		if (!strcmp(type, "debugfs")) {
 			fclose(fp);
-			strncpy(tracing_dir, dir, len - 1);
-			tracing_dir[len - 1] = '\0';
+			strscpy(tracing_dir, dir, len - 1);
 			return 0;
 		}
-	} while(1);
+	} while (1);
 
 	fclose(fp);
 	log(ALL, LOG_INFO, "Can't find debugfs\n");
-	return ENOENT;
+	return -ENOENT;
+}
+
+static int wait_access(char *path, int ms)
+{
+	int i;
+
+	for (i = 0; i < ms; i++) {
+		if (access(path, F_OK) == 0)
+			return 0;
+		usleep(1000);
+	}
+
+	log(ALL, LOG_WARNING, "%s failed, %s not created in %d ms\n",
+	    __func__, path, ms);
+	return -1;
 }
 
 static int open_trace(struct ras_events *ras, char *name, int flags)
 {
 	char fname[MAX_PATH + 1];
+	int rc;
 
-	strcpy(fname, ras->tracing);
-	strcat(fname, "/");
-	strcat(fname, name);
+	rc = strscpy(fname, ras->tracing, sizeof(fname));
+	if (rc < 0)
+		return rc;
+	rc = strscat(fname, "/", sizeof(fname));
+	if (rc < 0)
+		return rc;
+	rc = strscat(fname, name, sizeof(fname));
+	if (rc < 0)
+		return rc;
 
-	return open(fname, flags);
+	rc = wait_access(fname, 30000);
+	if (rc != 0) {
+		/* use -1 to keep same error value with open() */
+		return -1;
+	}
+
+	rc = open(fname, flags);
+	if (rc < 0) {
+		rc = -errno;
+
+		return -errno;
+	}
+
+	return rc;
 }
 
 static int get_tracing_dir(struct ras_events *ras)
@@ -120,11 +155,16 @@ static int get_tracing_dir(struct ras_events *ras)
 
 	get_debugfs_dir(ras->debugfs, sizeof(ras->debugfs));
 
-	strcpy(fname, ras->debugfs);
-	strcat(fname, "/tracing");
+	rc = strscpy(fname, ras->debugfs, sizeof(fname));
+	if (rc < 0)
+		return rc;
+	rc = strscat(fname, "/tracing", sizeof(fname));
+	if (rc < 0)
+		return rc;
+
 	dir = opendir(fname);
 	if (!dir)
-		return -1;
+		return -EINVAL;
 
 	for (entry = readdir(dir); entry; entry = readdir(dir)) {
 		if (strstr(entry->d_name, "instances")) {
@@ -134,21 +174,39 @@ static int get_tracing_dir(struct ras_events *ras)
 	}
 	closedir(dir);
 
-	strcpy(ras->tracing, ras->debugfs);
-	strcat(ras->tracing, "/tracing");
+	strscpy(ras->tracing, ras->debugfs, sizeof(ras->tracing));
+	strscat(ras->tracing, "/tracing", sizeof(ras->tracing));
 	if (has_instances) {
-		strcat(ras->tracing, "/instances/" TOOL_NAME);
-		rc = mkdir(ras->tracing, S_IRWXU);
+		rc = strscat(ras->tracing, "/instances/" TOOL_NAME,
+			     sizeof(ras->tracing));
+		if (rc < 0)
+			return rc;
+
+		rc = mkdir(ras->tracing, 0700);
 		if (rc < 0 && errno != EEXIST) {
 			log(ALL, LOG_INFO,
 			    "Unable to create " TOOL_NAME " instance at %s\n",
 			    ras->tracing);
-			return -1;
+			return -EINVAL;
 		}
 	}
 	return 0;
 }
 
+static bool is_disabled_event(char *group, char *event)
+{
+	char ras_event_name[MAX_PATH + 1];
+
+	snprintf(ras_event_name, sizeof(ras_event_name), "%s:%s",
+		 group, event);
+
+	if (choices_disable && strlen(choices_disable) != 0 &&
+	    strstr(choices_disable, ras_event_name)) {
+		return true;
+	}
+	return false;
+}
+
 /*
  * Tracing enable/disable code
  */
@@ -158,6 +216,9 @@ static int __toggle_ras_mc_event(struct ras_events *ras,
 	int fd, rc;
 	char fname[MAX_PATH + 1];
 
+	if (enable)
+		enable = is_disabled_event(group, event) ? 0 : 1;
+
 	snprintf(fname, sizeof(fname), "%s%s:%s\n",
 		 enable ? "" : "!",
 		 group, event);
@@ -166,10 +227,10 @@ static int __toggle_ras_mc_event(struct ras_events *ras,
 	fd = open_trace(ras, "set_event", O_RDWR | O_APPEND);
 	if (fd < 0) {
 		log(ALL, LOG_WARNING, "Can't open set_event\n");
-		return errno;
+		return -errno;
 	}
 
-	rc = write(fd, fname,strlen(fname));
+	rc = write(fd, fname, strlen(fname));
 	if (rc < 0) {
 		log(ALL, LOG_WARNING, "Can't write to set_event\n");
 		close(fd);
@@ -178,10 +239,10 @@ static int __toggle_ras_mc_event(struct ras_events *ras,
 	close(fd);
 	if (!rc) {
 		log(ALL, LOG_WARNING, "Nothing was written on set_event\n");
-		return EIO;
+		return -EIO;
 	}
 
-	log(ALL, LOG_INFO, "%s:%s event %s\n",
+	log(TERM, LOG_DEBUG, "%s:%s event %s\n",
 	    group, event,
 	    enable ? "enabled" : "disabled");
 
@@ -196,7 +257,7 @@ int toggle_ras_mc_event(int enable)
 	ras = calloc(1, sizeof(*ras));
 	if (!ras) {
 		log(TERM, LOG_ERR, "Can't allocate memory for ras struct\n");
-		return errno;
+		return -errno;
 	}
 
 	rc = get_tracing_dir(ras);
@@ -232,7 +293,7 @@ int toggle_ras_mc_event(int enable)
 #endif
 
 #ifdef HAVE_DISKERROR
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(5,18,0)
+#ifdef HAVE_BLK_RQ_ERROR
 	rc |= __toggle_ras_mc_event(ras, "block", "block_rq_error", enable);
 #else
 	rc |= __toggle_ras_mc_event(ras, "block", "block_rq_complete", enable);
@@ -243,12 +304,38 @@ int toggle_ras_mc_event(int enable)
 	rc |= __toggle_ras_mc_event(ras, "ras", "memory_failure_event", enable);
 #endif
 
+#ifdef HAVE_CXL
+	rc |= __toggle_ras_mc_event(ras, "cxl", "cxl_poison", enable);
+	rc |= __toggle_ras_mc_event(ras, "cxl", "cxl_aer_uncorrectable_error", enable);
+	rc |= __toggle_ras_mc_event(ras, "cxl", "cxl_aer_correctable_error", enable);
+	rc |= __toggle_ras_mc_event(ras, "cxl", "cxl_overflow", enable);
+	rc |= __toggle_ras_mc_event(ras, "cxl", "cxl_generic_event", enable);
+	rc |= __toggle_ras_mc_event(ras, "cxl", "cxl_general_media", enable);
+	rc |= __toggle_ras_mc_event(ras, "cxl", "cxl_dram", enable);
+	rc |= __toggle_ras_mc_event(ras, "cxl", "cxl_memory_module", enable);
+#endif
+
 free_ras:
 	free(ras);
-	return rc;
+	if (rc)
+		return -EINVAL;
+
+	return 0;
 }
 
-#if LINUX_VERSION_CODE < KERNEL_VERSION(5,18,0)
+static void setup_event_trigger(char *event)
+{
+	struct event_trigger trigger;
+
+	for (int i = 0; i < ARRAY_SIZE(event_triggers); i++) {
+		trigger = event_triggers[i];
+		if (!strcmp(event, trigger.name))
+			trigger.setup();
+	}
+}
+
+#ifdef HAVE_DISKERROR
+#ifndef HAVE_BLK_RQ_ERROR
 /*
  * Set kernel filter. libtrace doesn't provide an API for setting filters
  * in kernel, we have to implement it here.
@@ -263,10 +350,10 @@ static int filter_ras_mc_event(struct ras_events *ras, char *group, char *event,
 	fd = open_trace(ras, fname, O_RDWR | O_APPEND);
 	if (fd < 0) {
 		log(ALL, LOG_WARNING, "Can't open filter file\n");
-		return errno;
+		return -errno;
 	}
 
-	rc = write(fd, filter_str ,strlen(filter_str));
+	rc = write(fd, filter_str, strlen(filter_str));
 	if (rc < 0) {
 		log(ALL, LOG_WARNING, "Can't write to filter file\n");
 		close(fd);
@@ -275,12 +362,13 @@ static int filter_ras_mc_event(struct ras_events *ras, char *group, char *event,
 	close(fd);
 	if (!rc) {
 		log(ALL, LOG_WARNING, "Nothing was written on filter file\n");
-		return EIO;
+		return -EIO;
 	}
 
 	return 0;
 }
 #endif
+#endif
 
 /*
  * Tracing read code
@@ -304,7 +392,6 @@ static int get_pagesize(struct ras_events *ras, struct tep_handle *pevent)
 error:
 	close(fd);
 	return page_size;
-
 }
 
 static void parse_ras_data(struct pthread_data *pdata, struct kbuffer *kbuf,
@@ -327,9 +414,9 @@ static void parse_ras_data(struct pthread_data *pdata, struct kbuffer *kbuf,
 	trace_seq_init(&s);
 	tep_print_event(pdata->ras->pevent, &s, &record,
 			"%16s-%-5d [%03d] %s %6.1000d %s %s",
-                        TEP_PRINT_COMM, TEP_PRINT_PID, TEP_PRINT_CPU,
-                        TEP_PRINT_LATENCY, TEP_PRINT_TIME, TEP_PRINT_NAME,
-                        TEP_PRINT_INFO);
+			TEP_PRINT_COMM, TEP_PRINT_PID, TEP_PRINT_CPU,
+			TEP_PRINT_LATENCY, TEP_PRINT_TIME, TEP_PRINT_NAME,
+			TEP_PRINT_INFO);
 	trace_seq_do_printf(&s);
 	printf("\n");
 	fflush(stdout);
@@ -338,33 +425,53 @@ static void parse_ras_data(struct pthread_data *pdata, struct kbuffer *kbuf,
 
 static int get_num_cpus(struct ras_events *ras)
 {
-	return sysconf(_SC_NPROCESSORS_CONF);
-#if 0
-	char fname[MAX_PATH + 1];
-	int num_cpus = 0;
-	DIR		*dir;
-	struct dirent	*entry;
+	int cpus;
 
-	strcpy(fname, ras->debugfs);
-	strcat(fname, "/tracing/per_cpu/");
-	dir = opendir(fname);
-	if (!dir)
-		return -1;
-
-	for (entry = readdir(dir); entry; entry = readdir(dir)) {
-		if (strstr(entry->d_name, "cpu"))
-			num_cpus++;
-	}
-	closedir(dir);
-
-	return num_cpus;
-#endif
+	cpus = sysconf(_SC_NPROCESSORS_ONLN);
+	assert(cpus > 0);
+	return cpus;
 }
 
-static int read_ras_event_all_cpus(struct pthread_data *pdata,
-				   unsigned n_cpus)
+static int set_buffer_percent(struct ras_events *ras, int percent)
 {
-	unsigned size;
+	char buf[16];
+	ssize_t size;
+	int res = 0;
+	int fd;
+
+	fd = open_trace(ras, "buffer_percent", O_WRONLY);
+	if (fd >= 0) {
+		/* For the backward compatibility to the old kernels, do not return
+		 * if fail to set the buffer_percent.
+		 */
+		snprintf(buf, sizeof(buf), "%d", percent);
+		size = write(fd, buf, strlen(buf));
+		if (size <= 0) {
+			log(TERM, LOG_WARNING, "can't write to buffer_percent\n");
+			res = -EINVAL;
+		}
+		close(fd);
+	} else {
+		log(TERM, LOG_WARNING, "Can't open buffer_percent\n");
+		res = -EINVAL;
+	}
+
+	return res;
+}
+
+/*
+ * Kernel tracepoint had an incompatible change in 2019, causing polling
+ * tracepoints to fail. Rasdaemon can support both legacy and newer versions,
+ * with the help of a backup-compatibility legacy kernel mode.
+ *
+ * The LEGACY_KERNEL flag indicates the need to enable such code.
+ */
+#define LEGACY_KERNEL		255
+
+static int read_ras_event_all_cpus(struct pthread_data *pdata,
+				   unsigned int n_cpus)
+{
+	ssize_t size;
 	unsigned long long time_stamp;
 	void *data;
 	int ready, i, count_nready;
@@ -376,11 +483,6 @@ static int read_ras_event_all_cpus(struct pthread_data *pdata,
 	int warnonce[n_cpus];
 	char pipe_raw[PATH_MAX];
 	int legacy_kernel = 0;
-	int fd;
-	char buf[16];
-#if 0
-	int need_sleep = 0;
-#endif
 
 	memset(&warnonce, 0, sizeof(warnonce));
 
@@ -404,18 +506,8 @@ static int read_ras_event_all_cpus(struct pthread_data *pdata,
 	 * Set buffer_percent to 0 so that poll() will return immediately
 	 * when the trace data is available in the ras per_cpu trace pipe_raw
 	 */
-	fd = open_trace(pdata[0].ras, "buffer_percent", O_WRONLY);
-	if (fd >= 0) {
-		/* For the backward compatibility to the old kernels, do not return
-		 * if fail to set the buffer_percent.
-		 */
-		snprintf(buf, sizeof(buf), "0");
-		size = write(fd, buf, strlen(buf));
-		if (size <= 0)
-			log(TERM, LOG_WARNING, "can't write to buffer_percent\n");
-		close(fd);
-	} else
-		log(TERM, LOG_WARNING, "Can't open buffer_percent\n");
+	if (set_buffer_percent(pdata[0].ras, 0))
+		log(TERM, LOG_WARNING, "Set buffer_percent failed\n");
 
 	for (i = 0; i < (n_cpus + 1); i++)
 		fds[i].fd = -1;
@@ -425,7 +517,7 @@ static int read_ras_event_all_cpus(struct pthread_data *pdata,
 
 		/* FIXME: use select to open for all CPUs */
 		snprintf(pipe_raw, sizeof(pipe_raw),
-			"per_cpu/cpu%d/trace_pipe_raw", i);
+			 "per_cpu/cpu%d/trace_pipe_raw", i);
 
 		fds[i].fd = open_trace(pdata[0].ras, pipe_raw, O_RDONLY);
 		if (fds[i].fd < 0) {
@@ -452,13 +544,16 @@ static int read_ras_event_all_cpus(struct pthread_data *pdata,
 	if (pdata[0].ras->record_events) {
 		if (ras_mc_event_opendb(pdata[0].cpu, pdata[0].ras))
 			goto error;
+#ifdef HAVE_NON_STANDARD
+		if (ras_ns_add_vendor_tables(pdata[0].ras))
+			log(TERM, LOG_ERR, "Can't add vendor table\n");
+#endif
 	}
 
 	do {
 		ready = poll(fds, (n_cpus + 1), -1);
-		if (ready < 0) {
+		if (ready < 0)
 			log(TERM, LOG_WARNING, "poll\n");
-		}
 
 		/* check for the signal */
 		if (fds[n_cpus].revents & POLLIN) {
@@ -471,7 +566,7 @@ static int read_ras_event_all_cpus(struct pthread_data *pdata,
 			    fdsiginfo.ssi_signo == SIGTERM ||
 			    fdsiginfo.ssi_signo == SIGHUP ||
 			    fdsiginfo.ssi_signo == SIGQUIT) {
-				log(TERM, LOG_INFO, "Recevied signal=%d\n",
+				log(TERM, LOG_INFO, "Received signal=%d\n",
 				    fdsiginfo.ssi_signo);
 				goto  cleanup;
 			} else {
@@ -488,9 +583,6 @@ static int read_ras_event_all_cpus(struct pthread_data *pdata,
 					log(TERM, LOG_INFO,
 					    "Error on CPU %i\n", i);
 					warnonce[i]++;
-#if 0
-					need_sleep = 1;
-#endif
 				}
 			}
 			if (!(fds[i].revents & POLLIN)) {
@@ -505,6 +597,11 @@ static int read_ras_event_all_cpus(struct pthread_data *pdata,
 				kbuffer_load_subbuffer(kbuf, page);
 
 				while ((data = kbuffer_read_event(kbuf, &time_stamp))) {
+					if (kbuffer_curr_size(kbuf) < 0) {
+						log(TERM, LOG_ERR, "invalid kbuf data, discard\n");
+						break;
+					}
+
 					parse_ras_data(&pdata[i],
 						       kbuf, data, time_stamp);
 
@@ -515,10 +612,6 @@ static int read_ras_event_all_cpus(struct pthread_data *pdata,
 				count_nready++;
 			}
 		}
-#if 0
-		if (need_sleep)
-			sleep(POLLING_TIME);
-#else
 		/*
 		 * If we enable fallback mode, it will always be used, as
 		 * poll is still not working fine, IMHO
@@ -528,7 +621,6 @@ static int read_ras_event_all_cpus(struct pthread_data *pdata,
 			legacy_kernel = 1;
 			break;
 		}
-#endif
 	} while (1);
 
 	/* poll() is not supported. We need to fallback to the old way */
@@ -536,8 +628,12 @@ static int read_ras_event_all_cpus(struct pthread_data *pdata,
 	    "Old kernel detected. Stop listening and fall back to pthread way.\n");
 
 cleanup:
-	if (pdata[0].ras->record_events)
+	if (pdata[0].ras->record_events) {
+#ifdef HAVE_NON_STANDARD
+		ras_ns_finalize_vendor_tables();
+#endif
 		ras_mc_event_closedb(pdata[0].cpu, pdata[0].ras);
+	}
 
 error:
 	kbuffer_free(kbuf);
@@ -550,9 +646,9 @@ error:
 	}
 
 	if (legacy_kernel)
-		return -255;
-	else
-		return -1;
+		return LEGACY_KERNEL;
+
+	return -EINVAL;
 }
 
 static int read_ras_event(int fd,
@@ -573,7 +669,7 @@ static int read_ras_event(int fd,
 		size = read(fd, page, pdata->ras->page_size);
 		if (size < 0) {
 			log(TERM, LOG_WARNING, "read\n");
-			return -1;
+			return -EINVAL;
 		} else if (size > 0) {
 			kbuffer_load_subbuffer(kbuf, page);
 
@@ -625,19 +721,32 @@ static void *handle_ras_events_cpu(void *priv)
 
 	log(TERM, LOG_INFO, "Listening to events on cpu %d\n", pdata->cpu);
 	if (pdata->ras->record_events) {
+		pthread_mutex_lock(&pdata->ras->db_lock);
 		if (ras_mc_event_opendb(pdata->cpu, pdata->ras)) {
+			pthread_mutex_unlock(&pdata->ras->db_lock);
 			log(TERM, LOG_ERR, "Can't open database\n");
 			close(fd);
 			kbuffer_free(kbuf);
 			free(page);
 			return 0;
 		}
+#ifdef HAVE_NON_STANDARD
+		if (ras_ns_add_vendor_tables(pdata->ras))
+			log(TERM, LOG_ERR, "Can't add vendor table\n");
+#endif
+		pthread_mutex_unlock(&pdata->ras->db_lock);
 	}
 
 	read_ras_event(fd, pdata, kbuf, page);
 
-	if (pdata->ras->record_events)
+	if (pdata->ras->record_events) {
+		pthread_mutex_lock(&pdata->ras->db_lock);
+#ifdef HAVE_NON_STANDARD
+		ras_ns_finalize_vendor_tables();
+#endif
 		ras_mc_event_closedb(pdata->cpu, pdata->ras);
+		pthread_mutex_unlock(&pdata->ras->db_lock);
+	}
 
 	close(fd);
 	kbuffer_free(kbuf);
@@ -654,20 +763,20 @@ static int select_tracing_timestamp(struct ras_events *ras)
 	int fd, rc;
 	time_t uptime, now;
 	size_t size;
-	unsigned j1;
+	unsigned int j1;
 	char buf[4096];
 
 	/* Check if uptime is supported (kernel 3.10-rc1 or upper) */
 	fd = open_trace(ras, "trace_clock", O_RDONLY);
 	if (fd < 0) {
 		log(TERM, LOG_ERR, "Can't open trace_clock\n");
-		return -1;
+		return -EINVAL;
 	}
 	size = read(fd, buf, sizeof(buf));
 	close(fd);
 	if (!size) {
 		log(TERM, LOG_ERR, "trace_clock is empty!\n");
-		return -1;
+		return -EINVAL;
 	}
 
 	if (!strstr(buf, UPTIME)) {
@@ -702,7 +811,7 @@ static int select_tracing_timestamp(struct ras_events *ras)
 	fclose(fp);
 	if (rc <= 0) {
 		log(TERM, LOG_ERR, "Can't parse /proc/uptime!\n");
-		return -1;
+		return -EINVAL;
 	}
 	now = time(NULL);
 
@@ -712,29 +821,38 @@ static int select_tracing_timestamp(struct ras_events *ras)
 	return 0;
 }
 
+#define EVENT_DISABLED	1
+
 static int add_event_handler(struct ras_events *ras, struct tep_handle *pevent,
-			     unsigned page_size, char *group, char *event,
+			     unsigned int page_size, char *group, char *event,
 			     tep_event_handler_func func, char *filter_str, int id)
 {
 	int fd, size, rc;
 	char *page, fname[MAX_PATH + 1];
-	struct tep_event_filter * filter = NULL;
+	struct tep_event_filter *filter = NULL;
 
 	snprintf(fname, sizeof(fname), "events/%s/%s/format", group, event);
 
 	fd = open_trace(ras, fname, O_RDONLY);
 	if (fd < 0) {
-		log(TERM, LOG_ERR,
-		    "Can't get %s:%s traces. Perhaps this feature is not supported on your system.\n",
-		    group, event);
-		return errno;
+		if (fd == -ENOENT) {
+			log(TERM, LOG_ERR,
+			    "Feature %s:%s not supported on your system.\n",
+			    group, event);
+			return EVENT_DISABLED;
+		}
+
+		log(TERM, LOG_ERR, "Can't get %s:%s traces: %s\n",
+		    group, event, strerror(-fd));
+
+		return fd;
 	}
 
 	page = malloc(page_size);
 	if (!page) {
+		rc = -errno;
 		log(TERM, LOG_ERR, "Can't allocate page to read %s:%s format\n",
 		    group, event);
-		rc = errno;
 		close(fd);
 		return rc;
 	}
@@ -753,14 +871,14 @@ static int add_event_handler(struct ras_events *ras, struct tep_handle *pevent,
 		log(TERM, LOG_ERR, "Can't register event handler for %s:%s\n",
 		    group, event);
 		free(page);
-		return EINVAL;
+		return -EINVAL;
 	}
 
 	rc = tep_parse_event(pevent, page, size, group);
 	if (rc) {
 		log(TERM, LOG_ERR, "Can't parse event %s:%s\n", group, event);
 		free(page);
-		return EINVAL;
+		return -EINVAL;
 	}
 
 	if (filter_str) {
@@ -771,7 +889,7 @@ static int add_event_handler(struct ras_events *ras, struct tep_handle *pevent,
 			log(TERM, LOG_ERR,
 			    "Failed to allocate filter for %s/%s.\n", group, event);
 			free(page);
-			return EINVAL;
+			return -EINVAL;
 		}
 		rc = tep_filter_add_filter_str(filter, filter_str);
 		if (rc < 0) {
@@ -786,6 +904,12 @@ static int add_event_handler(struct ras_events *ras, struct tep_handle *pevent,
 
 	ras->filters[id] = filter;
 
+	if (is_disabled_event(group, event)) {
+		log(ALL, LOG_INFO, "Disabled %s:%s tracing from config\n",
+		    group, event);
+		return EVENT_DISABLED;
+	}
+
 	/* Enable RAS events */
 	rc = __toggle_ras_mc_event(ras, group, event, 1);
 	free(page);
@@ -793,19 +917,21 @@ static int add_event_handler(struct ras_events *ras, struct tep_handle *pevent,
 		log(TERM, LOG_ERR, "Can't enable %s:%s tracing\n",
 		    group, event);
 
-		return EINVAL;
+		return -EINVAL;
 	}
 
+	setup_event_trigger(event);
+
 	log(ALL, LOG_INFO, "Enabled event %s:%s\n", group, event);
 
 	return 0;
 }
 
-int handle_ras_events(int record_events)
+int handle_ras_events(int record_events, int enable_ipmitool)
 {
 	int rc, page_size, i;
 	int num_events = 0;
-	unsigned cpus;
+	unsigned int cpus;
 	struct tep_handle *pevent = NULL;
 	struct pthread_data *data = NULL;
 	struct ras_events *ras = NULL;
@@ -816,7 +942,7 @@ int handle_ras_events(int record_events)
 	ras = calloc(1, sizeof(*ras));
 	if (!ras) {
 		log(TERM, LOG_ERR, "Can't allocate memory for ras struct\n");
-		return errno;
+		return -errno;
 	}
 
 	rc = get_tracing_dir(ras);
@@ -826,15 +952,13 @@ int handle_ras_events(int record_events)
 	}
 
 	rc = select_tracing_timestamp(ras);
-	if (rc < 0) {
-		log(TERM, LOG_ERR, "Can't select a timestamp for tracing\n");
-		goto err;
-	}
+	if (rc < 0)
+		log(TERM, LOG_ERR, "Can't select a timestamp for tracing. Using default\n");
 
 	pevent = tep_alloc();
 	if (!pevent) {
 		log(TERM, LOG_ERR, "Can't allocate pevent\n");
-		rc = errno;
+		rc = -errno;
 		goto err;
 	}
 
@@ -844,6 +968,10 @@ int handle_ras_events(int record_events)
 	ras->page_size = page_size;
 	ras->record_events = record_events;
 
+#ifdef HAVE_MEMORY_ROW_CE_PFA
+	ras_row_account_init();
+#endif
+
 #ifdef HAVE_MEMORY_CE_PFA
 	/* FIXME: enable memory isolation unconditionally */
 	ras_page_account_init();
@@ -853,16 +981,17 @@ int handle_ras_events(int record_events)
 			       ras_mc_event_handler, NULL, MC_EVENT);
 	if (!rc)
 		num_events++;
-	else
+	else if (rc != EVENT_DISABLED)
 		log(ALL, LOG_ERR, "Can't get traces from %s:%s\n",
 		    "ras", "mc_event");
 
 #ifdef HAVE_AER
+	ras_aer_handler_init(enable_ipmitool);
 	rc = add_event_handler(ras, pevent, page_size, "ras", "aer_event",
 			       ras_aer_event_handler, NULL, AER_EVENT);
 	if (!rc)
 		num_events++;
-	else
+	else if (rc != EVENT_DISABLED && rc != ENOENT)
 		log(ALL, LOG_ERR, "Can't get traces from %s:%s\n",
 		    "ras", "aer_event");
 #endif
@@ -872,7 +1001,7 @@ int handle_ras_events(int record_events)
 			       ras_non_standard_event_handler, NULL, NON_STANDARD_EVENT);
 	if (!rc)
 		num_events++;
-	else
+	else if (rc != EVENT_DISABLED)
 		log(ALL, LOG_ERR, "Can't get traces from %s:%s\n",
 		    "ras", "non_standard_event");
 #endif
@@ -882,7 +1011,7 @@ int handle_ras_events(int record_events)
 			       ras_arm_event_handler, NULL, ARM_EVENT);
 	if (!rc)
 		num_events++;
-	else
+	else if (rc != EVENT_DISABLED)
 		log(ALL, LOG_ERR, "Can't get traces from %s:%s\n",
 		    "ras", "arm_event");
 #endif
@@ -895,7 +1024,7 @@ int handle_ras_events(int record_events)
 
 #ifdef HAVE_MCE
 	rc = register_mce_handler(ras, cpus);
-	if (rc)
+	if (rc && rc != -ENOENT)
 		log(ALL, LOG_INFO, "Can't register mce handler\n");
 	if (ras->mce_priv) {
 		rc = add_event_handler(ras, pevent, page_size,
@@ -916,7 +1045,7 @@ int handle_ras_events(int record_events)
 		/* tell kernel we are listening, so don't printk to console */
 		(void)open("/sys/kernel/debug/ras/daemon_active", 0);
 		num_events++;
-	} else
+	} else if (rc != EVENT_DISABLED)
 		log(ALL, LOG_ERR, "Can't get traces from %s:%s\n",
 		    "ras", "extlog_mem_event");
 #endif
@@ -933,19 +1062,19 @@ int handle_ras_events(int record_events)
 			       ras_devlink_event_handler, filter_str, DEVLINK_EVENT);
 	if (!rc)
 		num_events++;
-	else
+	else if (rc != EVENT_DISABLED)
 		log(ALL, LOG_ERR, "Can't get traces from %s:%s\n",
 		    "devlink", "devlink_health_report");
 #endif
 
 #ifdef HAVE_DISKERROR
-#if LINUX_VERSION_CODE >= KERNEL_VERSION(5,18,0)
+#ifdef HAVE_BLK_RQ_ERROR
 	rc = add_event_handler(ras, pevent, page_size, "block",
 			       "block_rq_error", ras_diskerror_event_handler,
 				NULL, DISKERROR_EVENT);
 	if (!rc)
 		num_events++;
-	else
+	else if (rc != EVENT_DISABLED)
 		log(ALL, LOG_ERR, "Can't get traces from %s:%s\n",
 		    "block", "block_rq_error");
 #else
@@ -956,7 +1085,7 @@ int handle_ras_events(int record_events)
 					NULL, DISKERROR_EVENT);
 		if (!rc)
 			num_events++;
-		else
+		else if (rc != EVENT_DISABLED)
 			log(ALL, LOG_ERR, "Can't get traces from %s:%s\n",
 			    "block", "block_rq_complete");
 	}
@@ -968,23 +1097,88 @@ int handle_ras_events(int record_events)
 			       ras_memory_failure_event_handler, NULL, MF_EVENT);
 	if (!rc)
 		num_events++;
-	else
+	else if (rc != EVENT_DISABLED)
 		log(ALL, LOG_ERR, "Can't get traces from %s:%s\n",
 		    "ras", "memory_failure_event");
 #endif
 
+#ifdef HAVE_CXL
+	rc = add_event_handler(ras, pevent, page_size, "cxl", "cxl_poison",
+			       ras_cxl_poison_event_handler, NULL, CXL_POISON_EVENT);
+	if (!rc)
+		num_events++;
+	else if (rc != EVENT_DISABLED)
+		log(ALL, LOG_ERR, "Can't get traces from %s:%s\n",
+		    "cxl", "cxl_poison");
+
+	rc = add_event_handler(ras, pevent, page_size, "cxl", "cxl_aer_uncorrectable_error",
+			       ras_cxl_aer_ue_event_handler, NULL, CXL_AER_UE_EVENT);
+	if (!rc)
+		num_events++;
+	else if (rc != EVENT_DISABLED)
+		log(ALL, LOG_ERR, "Can't get traces from %s:%s\n",
+		    "cxl", "cxl_aer_uncorrectable_error");
+
+	rc = add_event_handler(ras, pevent, page_size, "cxl", "cxl_aer_correctable_error",
+			       ras_cxl_aer_ce_event_handler, NULL, CXL_AER_CE_EVENT);
+	if (!rc)
+		num_events++;
+	else if (rc != EVENT_DISABLED)
+		log(ALL, LOG_ERR, "Can't get traces from %s:%s\n",
+		    "cxl", "cxl_aer_correctable_error");
+
+	rc = add_event_handler(ras, pevent, page_size, "cxl", "cxl_overflow",
+			       ras_cxl_overflow_event_handler, NULL, CXL_OVERFLOW_EVENT);
+	if (!rc)
+		num_events++;
+	else if (rc != EVENT_DISABLED)
+		log(ALL, LOG_ERR, "Can't get traces from %s:%s\n",
+		    "cxl", "cxl_overflow");
+
+	rc = add_event_handler(ras, pevent, page_size, "cxl", "cxl_generic_event",
+			       ras_cxl_generic_event_handler, NULL, CXL_GENERIC_EVENT);
+	if (!rc)
+		num_events++;
+	else if (rc != EVENT_DISABLED)
+		log(ALL, LOG_ERR, "Can't get traces from %s:%s\n",
+		    "cxl", "cxl_generic_event");
+
+	rc = add_event_handler(ras, pevent, page_size, "cxl", "cxl_general_media",
+			       ras_cxl_general_media_event_handler, NULL, CXL_GENERAL_MEDIA_EVENT);
+	if (!rc)
+		num_events++;
+	else if (rc != EVENT_DISABLED)
+		log(ALL, LOG_ERR, "Can't get traces from %s:%s\n",
+		    "cxl", "cxl_general_media");
+
+	rc = add_event_handler(ras, pevent, page_size, "cxl", "cxl_dram",
+			       ras_cxl_dram_event_handler, NULL, CXL_DRAM_EVENT);
+	if (!rc)
+		num_events++;
+	else if (rc != EVENT_DISABLED)
+		log(ALL, LOG_ERR, "Can't get traces from %s:%s\n",
+		    "cxl", "cxl_dram");
+
+	rc = add_event_handler(ras, pevent, page_size, "cxl", "cxl_memory_module",
+			       ras_cxl_memory_module_event_handler, NULL, CXL_MEMORY_MODULE_EVENT);
+	if (!rc)
+		num_events++;
+	else if (rc != EVENT_DISABLED)
+		log(ALL, LOG_ERR, "Can't get traces from %s:%s\n",
+		    "cxl", "memory_module");
+#endif
+
 	if (!num_events) {
 		log(ALL, LOG_INFO,
-		    "Failed to trace all supported RAS events. Aborting.\n");
+		    "Failed to trace any supported RAS events. Aborting.\n");
 		rc = -EINVAL;
 		goto err;
 	}
 
-	data = calloc(sizeof(*data), cpus);
+	data = calloc(cpus, sizeof(*data));
 	if (!data)
 		goto err;
 
-
 	for (i = 0; i < cpus; i++) {
 		data[i].ras = ras;
 		data[i].cpu = i;
@@ -992,19 +1186,26 @@ int handle_ras_events(int record_events)
 	rc = read_ras_event_all_cpus(data, cpus);
 
 	/* Poll doesn't work on this kernel. Fallback to pthread way */
-	if (rc == -255) {
+	if (rc == LEGACY_KERNEL) {
+		if (pthread_mutex_init(&ras->db_lock, NULL) != 0) {
+			log(SYSLOG, LOG_INFO, "sqlite db lock init has failed\n");
+			goto err;
+		}
+
 		log(SYSLOG, LOG_INFO,
-		"Opening one thread per cpu (%d threads)\n", cpus);
+		    "Opening one thread per cpu (%d threads)\n", cpus);
 		for (i = 0; i < cpus; i++) {
 			rc = pthread_create(&data[i].thread, NULL,
-					handle_ras_events_cpu,
+					    handle_ras_events_cpu,
 					(void *)&data[i]);
 			if (rc) {
 				log(SYSLOG, LOG_INFO,
-				"Failed to create thread for cpu %d. Aborting.\n",
+				    "Failed to create thread for cpu %d. Aborting.\n",
 				i);
 				while (--i)
 					pthread_cancel(data[i].thread);
+
+				pthread_mutex_destroy(&ras->db_lock);
 				goto err;
 			}
 		}
@@ -1012,6 +1213,7 @@ int handle_ras_events(int record_events)
 		/* Wait for all threads to complete */
 		for (i = 0; i < cpus; i++)
 			pthread_join(data[i].thread, NULL);
+		pthread_mutex_destroy(&ras->db_lock);
 	}
 
 	log(SYSLOG, LOG_INFO, "Huh! something got wrong. Aborting.\n");
@@ -1033,5 +1235,9 @@ err:
 #ifdef HAVE_CPU_FAULT_ISOLATION
 	cpu_infos_free();
 #endif
+
+#ifdef HAVE_MEMORY_ROW_CE_PFA
+	row_record_infos_free();
+#endif
 	return rc;
 }
